Package: CHETAH
Title: Fast and accurate scRNA-seq cell type identification
Type: Package
Version: 1.28.0
Date: 2025-10-14
Authors@R: c(person("Jurrian", "de Kanter", email = "jurriandekanter@gmail.com",  role = c("aut", "cre")),
             person("Philip", "Lijnzaad", email = "p.lijnzaad@prinsesmaximacentrum.nl", role = c("aut")))
Description: CHETAH (CHaracterization of cEll Types Aided by Hierarchical classification) is an accurate, selective and fast scRNA-seq classifier.
	  Classification is guided by a reference dataset,
    preferentially also a scRNA-seq dataset. By hierarchical clustering of the reference data, CHETAH creates
	  a classification tree that enables a step-wise, top-to-bottom classification. Using a novel stopping rule,
	  CHETAH classifies the input cells to the cell types of the references and to "intermediate types": more general
	  classifications that ended in an intermediate node of the tree.
Imports: shiny, plotly, pheatmap, bioDist, dendextend, cowplot,
        corrplot, grDevices, stats, graphics, reshape2, S4Vectors,
        SummarizedExperiment
Depends: R (>= 4.2), ggplot2, SingleCellExperiment
License: file LICENSE
Encoding: UTF-8
biocViews: Classification, RNASeq, SingleCell, Clustering,
        GeneExpression, ImmunoOncology
RoxygenNote: 7.2.0
Suggests: knitr, rmarkdown, Matrix, testthat, vdiffr
VignetteBuilder: knitr
LazyData: false
BugReports: https://github.com/jdekanter/CHETAH
URL: https://github.com/jdekanter/CHETAH
git_url: https://git.bioconductor.org/packages/CHETAH
git_branch: RELEASE_3_23
git_last_commit: e5eb1b6
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:12:07 UTC; biocbuild
Author: Jurrian de Kanter [aut, cre],
  Philip Lijnzaad [aut]
Maintainer: Jurrian de Kanter <jurriandekanter@gmail.com>
